/*---------------------------------------------------------------------------*
  Project:  MicTest - sample1
  File:     main.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#include <nitro.h>
#include "nitro/mictest.h"

//---------------------------------------------------------------------------
//    V-Blank interrupt process--
static void VBlankIntr(void)
{
    OS_SetIrqCheckFlag( OS_IE_V_BLANK );
}

//=============================================================================
//  Main
//=============================================================================
static u8 s_Work[ MICTEST_SIZE ] ATTRIBUTE_ALIGN( 32 );

void NitroMain()
{
    OS_Init();
    GX_Init();
    GX_DispOff();
    GXS_DispOff();

// V-Blank interrupt settings--
    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)GX_VBlankIntr(TRUE);

//---- Clear VRAM
    GX_SetBankForLCDC(GX_VRAM_LCDC_ALL);
    MI_CpuClearFast((void *)HW_LCDC_VRAM, HW_LCDC_VRAM_SIZE);
    (void)GX_DisableBankForLCDC();

// Initialize tick system
    OS_InitTick();
// Initialize alarm system
    OS_InitAlarm();

    OS_WaitVBlankIntr();
    GX_DispOn();
    GXS_DispOn();

    while(1)
    {
        mictest_init( s_Work );
        mictest_start();

        while(1)
        {
            u16 pad;
            int lvl = mictest_main();
            OS_TPrintf("level=%d\n",lvl);
            OS_WaitVBlankIntr();
            pad = PAD_Read();
            if (pad & PAD_BUTTON_B)
            {
                OS_TPrintf("stop\n");
                mictest_end();
                break;
            }
        }

        while(1)
        {
            u16 pad;
            OS_WaitVBlankIntr();
            pad = PAD_Read();
            if (pad & PAD_BUTTON_A)
            {
                OS_TPrintf("start\n");
                break;
            }
        }
    }
}
