/*---------------------------------------------------------------------------*
  Project:  MicTest - sample2
  File:     canvas_dn_obj.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include <nnsys/g2d.h>

#include "canvas_dn_obj.h"
#include "file.h"
#include "memory.h"
#include "canvas.h"

//--------------------------------------------------------------
#define  NUM_OF_OAM        128
#define  CANVAS_WIDTH       10       // Width of character rendering area (in character units)
#define  CANVAS_HEIGHT       4       // Height of character rendering area  (in character units)
#define  TEXT_HSPACE         1       // Amount of space between characters when rendering strings (in pixels)
#define  NUM_OF_OAM_DN       3

enum
{
    CELL0_NUMBER=0,
    CELL_MAX
};

//--------------------------------------------------------------
static  NNSG2dOamManagerInstance    gOamManager;
static  void*                       pCelSubFile;        // File for cells
static  NNSG2dCellDataBank*         pCellSubBank;       // Cell bank
static  NNSG2dCharCanvas            gCanvas[CELL_MAX];  // CharCanvas
static  NNSG2dCellData*             CanvasCell[CELL_MAX];
static  NNSG2dRendererInstance      gRenderer;          // For characters
static  NNSG2dRenderSurface         gSurface;           // For characters
static  NNSG2dImageProxy            gImageProxy;        // For characters
static  NNSG2dImagePaletteProxy     gPaletteProxy;      // For characters
static  GXOamAttr                   SubOamBuffer[NUM_OF_OAM_DN];
static  void*  font;

//--------------------------------------------------------------
static const wchar_t ModoruStrings[] = L"ǂ";
static const int cell_offset[CELL_MAX]={ 0x100 };
static const NNSG2dFVec2 ShitaPos=
{
    (128-45) *FX32_ONE, (96+69) *FX32_ONE        // Underlay position
};
static const NNSG2dFVec2 cellpos[CELL_MAX]=
{
    {(128-45+4) *FX32_ONE, (96+67-3) *FX32_ONE},// Character position
};

//--------------------------------------------------------------

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static BOOL CallBackAddOam( const GXOamAttr* pOam, u16 affineIndex, BOOL bDoubleAffine )
{
#pragma unused( bDoubleAffine )
    SDK_NULL_ASSERT( pOam );
    return NNS_G2dEntryOamManagerOamWithAffineIdx( &gOamManager, pOam, affineIndex );
}
/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static u16 CallBackAddAffine( const MtxFx22* mtx )
{
    SDK_NULL_ASSERT( mtx );
    return NNS_G2dEntryOamManagerAffine( &gOamManager, mtx );
}
/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static void InitOamManager(NNSG2dOamManagerInstance*   pOamManager)
{
    BOOL result;
    SDK_NULL_ASSERT( pOamManager );

    NNS_G2dInitOamManagerModule();

    result = NNS_G2dGetNewOamManagerInstanceAsFastTransferMode(
                pOamManager, 0, NUM_OF_OAM, NNS_G2D_OAMTYPE_SUB );
    SDK_ASSERT( result );
}
/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static void InitRenderer(
                NNSG2dRendererInstance*     pRenderer,
                NNSG2dRenderSurface*        pSurface,
                NNSG2dImageProxy*           pImageProxy,
                NNSG2dImagePaletteProxy*    pPaletteProxy
)
{
    // Initialize ImageProxy with a dummy address
    NNS_G2dInitImageProxy( pImageProxy );
    NNS_G2dSetImageLocation( pImageProxy, NNS_G2D_VRAM_TYPE_2DSUB, NULL );

    // Initialize PaletteProxy with a dummy address
    NNS_G2dInitImagePaletteProxy( pPaletteProxy );
    NNS_G2dSetImagePaletteLocation( pPaletteProxy, NNS_G2D_VRAM_TYPE_2DSUB, NULL );

    // Initialize the surface
    NNS_G2dInitRenderSurface( pSurface );

    pSurface->pFuncOamRegister          = CallBackAddOam;
    pSurface->pFuncOamAffineRegister    = CallBackAddAffine;
    pSurface->type                      = NNS_G2D_SURFACETYPE_SUB2D;

    // Initialize the renderer
    NNS_G2dInitRenderer( pRenderer );
    NNS_G2dAddRendererTargetSurface( pRenderer, pSurface );
    NNS_G2dSetRendererImageProxy( pRenderer, pImageProxy, pPaletteProxy );
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static void LoadObj()
{
    void*                   pChrFile;
    BOOL  ret;
    NNSG2dCharacterData*    pChrData;
    void*                   pPltFile;
    NNSG2dPaletteData*      pPltData;
    NNSG2dImageProxy        sImageProxy;
    NNSG2dImagePaletteProxy sPaletteProxy;

    NNS_G2dInitImageProxy( &sImageProxy );
    NNS_G2dInitImagePaletteProxy( &sPaletteProxy );

    pChrFile = LoadFile("E11d_obj.NCGR");
    ret = NNS_G2dGetUnpackedCharacterData(pChrFile, &pChrData);
    SDK_ASSERT(ret);
    NNS_G2dLoadImage1DMapping( pChrData, 0, NNS_G2D_VRAM_TYPE_2DSUB, &sImageProxy);

    pPltFile = LoadFile("E11d_obj.NCLR");
    ret = NNS_G2dGetUnpackedPaletteData  (pPltFile, &pPltData);
    SDK_ASSERT(ret);
    NNS_G2dLoadPalette       ( pPltData, 0, NNS_G2D_VRAM_TYPE_2DSUB, &sPaletteProxy);

    mem_free(pPltFile);
    mem_free(pChrFile);

    pCelSubFile = LoadFile("E11d_obj.NCER");
    ret = NNS_G2dGetUnpackedCellBank(pCelSubFile, &pCellSubBank);
    SDK_ASSERT(ret);
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static void InitCanvas(void)
{
    int i;
    int cOffset;
    GXCharFmt16* const pCharBase = (GXCharFmt16*)G2S_GetOBJCharPtr();
    const NNSG2dOBJVramMode vramMode = NNS_G2D_OBJVRAMMODE_128K;
    const int x                 = 0;
    const int y                 = 0;
    const int priority          = 0;
    const GXOamMode mode        = GX_OAM_MODE_NORMAL;
    const BOOL mosaic           = FALSE;
    const GXOamEffect effect    = GX_OAM_EFFECT_AFFINE_DOUBLE;
    const GXOamColorMode color  = GX_OAM_COLORMODE_16;
    const int cParam            = 0;
    const BOOL makeBR           = FALSE;
    size_t szByte;

    for (i=0;i<CELL_MAX;i++)
    {
        cOffset = cell_offset[i];
        // CharCanvas initialization
        NNS_G2dCharCanvasInitForOBJ1D(
            &gCanvas[i],
            pCharBase + (cOffset << vramMode),
            CANVAS_WIDTH,
            CANVAS_HEIGHT,
            NNS_G2D_CHARA_COLORMODE_16
        );
        
        // Build cell
        szByte         = NNS_G2dCharCanvasCalcCellDataSize1D(&gCanvas[i], makeBR);
        
        // Allocate memory for cell data
        CanvasCell[i] = (NNSG2dCellData*)mem_alloc(szByte, 4);
        SDK_NULL_ASSERT( CanvasCell[i] );

        NNS_G2dCharCanvasMakeCell1D(
            CanvasCell[i],
            &gCanvas[i],
            x, y,
            priority, mode, mosaic, effect, color,
            cell_offset[i],
            cParam, vramMode, makeBR
        );
    }
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void DnObjEnd()
{
    mem_free(CanvasCell[CELL0_NUMBER]);
    mem_free(pCelSubFile);
}

//-----------------------------------------------------------------------------
// Feature: Calculates the width of one line of a string.
// Arguments: space  - Character spacing
//        txt    - String
// Return values: int    - Width of 1 line
//-----------------------------------------------------------------------------
static int  calcWidth(int space, const wchar_t* txt)
{
    int        width;
    int        i;

    width = 0;
    for (i = 0; *txt; i++)
    {
        if (*txt == L'\n')
        {
            break;
        }
        width += NNS_G2dFontGetCharWidth(Retfont(), *txt) + space;
        txt++;
    }
    return  (i) ?  (width - space) : 0;
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static void DrawText(const wchar_t* moji, const int width, const int height, const int canNo)
{
    int x;
    int num;
    int y = (height *8 -NNS_G2dFontGetCellHeight(Retfont())) /2 + 2;
    wchar_t* code = (wchar_t*)moji;
    const int color = 1;

    num = calcWidth(TEXT_HSPACE, code);// Calculate the number of characters
    x = ((width *8 -num) /2)+8;

    while(1)
    {
        if (!*code)
        {
            break;
        }

        num = NNS_G2dCharCanvasDrawChar(
            &gCanvas[canNo],
            Retfont(),
            x, y,
            color,
            *code
        );
        x += num +TEXT_HSPACE;
        code++;
    }
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void DnObjInit()
{
    InitOamManager( &gOamManager );

    GXS_SetOBJVRamModeChar(GX_OBJVRAMMODE_CHAR_1D_128K);
    GXS_LoadOBJPltt(GetObjPalette(), 0, OBJ_PAL_SIZE);

    InitRenderer( &gRenderer, &gSurface, &gImageProxy, &gPaletteProxy );
    LoadObj();
    InitCanvas();

    DrawText(ModoruStrings, 8, 4, CELL0_NUMBER);
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void FontCreate()
{
    font = LoadFile("lc_m.NFTR");
    NNS_G2dFontInitAuto(Retfont(), font);
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void FontRelease()
{
    mem_free(font);
}

/*---------------------------------------------------------------------------*
    Render "Return"
 *---------------------------------------------------------------------------*/
void DnMojiDraw()
{
    int i;

    for(i=0;i<CELL_MAX;i++)
    {
    // Render the CharCanvas cell
        NNS_G2dBeginRendering( &gRenderer );
        {
            NNS_G2dTranslate( cellpos[i].x, cellpos[i].y, 0 );
            NNS_G2dScale(FX32_ONE, FX32_ONE, FX32_ONE);
            NNS_G2dDrawCell( CanvasCell[i] );
        }
        NNS_G2dEndRendering();
    }
}
//---------------------------------------------------------------------------
static u16  DrawSubSub(const u16 num, const NNSG2dFVec2  pos, const u16 offset)
{
    const NNSG2dCellData*   pCell;
    u16  numOam;

    SDK_ASSERT(offset < NUM_OF_OAM_DN);

    pCell = NNS_G2dGetCellDataByIdx(pCellSubBank, num);
    SDK_NULL_ASSERT( pCell );

    numOam = NNS_G2dMakeCellToOams(
        &SubOamBuffer[offset],// output Oam buffer
        NUM_OF_OAM_DN,// output buffer length
        pCell,        // Cell to output
        NULL,
        &pos,
        0,            // Affine Index
        FALSE);       // double affine?
    SDK_ASSERT( numOam < NUM_OF_OAM_DN );

    return  numOam;
}
/*---------------------------------------------------------------------------*
    Display the underlay OBJ
        number - 0 - OBJ that is not pressed
                 1 - Pressed
 *---------------------------------------------------------------------------*/
void DnEntryOam(const u16 number)
{
    u16  numOam;
    BOOL  ret;

    numOam  = DrawSubSub(number, ShitaPos, 0);
    ret = NNS_G2dEntryOamManagerOam(&gOamManager, SubOamBuffer, numOam);
    SDK_ASSERT(ret);
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void DnDrawManager()
{
    NNS_G2dApplyOamManagerToHW  ( &gOamManager );
    NNS_G2dResetOamManagerBuffer( &gOamManager );
}
