/*---------------------------------------------------------------------------*
  Project:  MicTest - sample2
  File:     cell.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include <nnsys/g2d/g2d_Load.h>
#include "cell.h"
#include "file.h"
#include "memory.h"

#define  GX_SCR_MAX  1

//============================================================================
// static variables
//============================================================================
static  void*   pObj_cell_list[GX_SCR_MAX];
static  NNSG2dCellDataBank* pCellBank[GX_SCR_MAX];

/*-----------------------------------------------------
-----------------------------------------------------*/
const NNSG2dCellData* RetCellData(const u16 no, const s16 type)
{
    return NNS_G2dGetCellDataByIdx( pCellBank[type], no );
}

/*-----------------------------------------------------
Function: Cell_SetCellData
Description: Sets a cell's cell data
Arguments: const char *cell_file_name  - Cell file name
              const int scr_type -          Screen type (either main or sub)
Returns: None
-----------------------------------------------------*/
void Cell_SetCellData(const char *cell_file_name, const int scr_type)
{
    pObj_cell_list[scr_type] = LoadFile(cell_file_name);
    SDK_ASSERTMSG(pObj_cell_list[scr_type], "load failed");
    NNS_G2dGetUnpackedCellBank(pObj_cell_list[scr_type], &pCellBank[scr_type]);
}

/*-----------------------------------------------------
Function: Cell_UnloadCellFiles
Description: Frees memory that contains an OBJ cell file
Arguments: GXSCRTYPE scr_type  -  Either main screen or sub screen
Returns: None
-----------------------------------------------------*/
void Cell_UnloadCellFiles(const int scr_type)
{
    if (scr_type == GE_BOTH)
    {
        if (pObj_cell_list[GE_MAIN])
        {
            mem_free(pObj_cell_list[GE_MAIN]);
        }
        if (pObj_cell_list[GE_SUB])
        {
            mem_free(pObj_cell_list[GE_SUB]);
        }
        return;
    }

    if (pObj_cell_list[scr_type])
    {
        mem_free(pObj_cell_list[scr_type]);
    }
}

/*-----------------------------------------------------
-----------------------------------------------------*/
int SetCellDataToOAM(GXOamAttr* oam_inp, const cell* pCell)
{
    int j;
    int ret = 0;
    NNSG2dCellOAMAttrData*  pOBJ;
    GXOamAttr* oam = oam_inp;

    pOBJ = pCell->celldata->pOamAttrArray;
    for( j = 0; j < pCell->celldata->numOAMAttrs; j++ )
    {
        oam->attr0 = pOBJ->attr0;
        oam->attr1 = pOBJ->attr1;
        oam->attr2 = pOBJ->attr2;
        oam++;
        pOBJ++;
        ret++;
    }

    oam = oam_inp;
    for( j = 0; j < pCell->celldata->numOAMAttrs; j++ )
    {
        oam->x += pCell->x;
        oam->y += pCell->y;
        oam++;
    }
    return ret;
}
