/*---------------------------------------------------------------------------*
  Project:  MicTest - sample2
  File:     main.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include <nnsys/g2d.h>
#include "nitro/mictest.h"
#include "memory.h"
#include "tp.h"
#include "canvas_up_bg.h"
#include "canvas_dn_bg.h"
#include "canvas_dn_obj.h"
#include "obj.h"

extern void BgInit(void);
extern void fadeout(void);
extern void fadein(void);

void ReadyUpdateCallback(void*);

static OSAlarm sReadyAlarm;
static u8 ReadyCount;

/*---------------------------------------------------------------------------*
    VBlank interrupt process
 *---------------------------------------------------------------------------*/
static void VBlankIntr(void)
{
    OS_SetIrqCheckFlag( OS_IE_V_BLANK );
}

/*---------------------------------------------------------------------------*
    Main
 *---------------------------------------------------------------------------*/
void NitroMain(void)
{
    static void* s_Work;

    OS_Init();
    GX_Init();
    GX_DispOff();
    GXS_DispOff();

    FS_Init(3);
    mem_init();

// VBlank interrupt settings--
    OS_SetIrqFunction(OS_IE_V_BLANK, VBlankIntr);
    (void)OS_EnableIrqMask(OS_IE_V_BLANK);
    (void)OS_EnableIrq();
    (void)GX_VBlankIntr(TRUE);

// Clear VRAM
    GX_SetBankForLCDC(GX_VRAM_LCDC_ALL);
    MI_CpuClearFast((void *)HW_LCDC_VRAM, HW_LCDC_VRAM_SIZE);
    (void)GX_DisableBankForLCDC();

// Initialize tick system
    OS_InitTick();
// Initialize alarm system
    OS_InitAlarm();

    GX_SetBankForBG    (GX_VRAM_BG_128_A);
    GX_SetBankForOBJ   (GX_VRAM_OBJ_128_B);
    GX_SetBankForSubBG (GX_VRAM_SUB_BG_128_C);
    GX_SetBankForSubOBJ(GX_VRAM_SUB_OBJ_128_D);
    GX_SetBankForOBJExtPltt   (GX_VRAM_OBJEXTPLTT_0_G);
    GX_SetBankForSubOBJExtPltt(GX_VRAM_SUB_OBJEXTPLTT_0_I);
    BgInit();
    FontCreate();
    DnObjInit();
    BgUpInitCanvas();

    GX_SetVisiblePlane (GX_GetVisiblePlane()  | GX_PLANEMASK_BG0 | GX_PLANEMASK_OBJ);
    GXS_SetVisiblePlane(GXS_GetVisiblePlane() | GX_PLANEMASK_BG0 | GX_PLANEMASK_OBJ);
    OS_WaitVBlankIntr();
    GX_DispOn();
    GXS_DispOn();

    tp_init();
    while(1)
    {
        BOOL end_flag = FALSE;
        int lvl = 0;

        ObjInit();
        BgDnInitCanvas();
        tp_start();

        s_Work = mem_alloc(MICTEST_SIZE, 32);
        mictest_init( s_Work );
        mictest_start();

        // Alarm settings for updating progress on the standby screen
        ReadyCount = 0;
        OS_CreateAlarm(&sReadyAlarm);
        OS_SetPeriodicAlarm(&sReadyAlarm, OS_GetTick()+READY_UPDATE_TICKS, READY_UPDATE_TICKS, &ReadyUpdateCallback, NULL);

        // Main loop for the mic test
        while(1)
        {
            u16 pad = PAD_Read();
            u16 tp = TOUCH_NONE;

            // Move from the standby screen to a screen in which the mic test can be performed
            if (mictest_IsReady())
            {
                //Cancel the alarm for updating progress on the standby screen
                ReadyCount = 0;
                OS_CancelAlarm(&sReadyAlarm);

                //Change the message for the screen at which the mic test can be performed
                DnBgClr();
                BgDnMicStartDrawText();
            }

            lvl = mictest_main();
            ObjMain(lvl);
            tp = tp_main();
            if ((tp == TOUCH_OFF_TRG) || (pad & PAD_BUTTON_B))
            {
                fadeout();
                end_flag = TRUE;
            }
            DnMojiDraw();
            DnEntryOam((tp==TOUCH_ON_TRG) ? (u16)1 : (u16)0);

            OS_WaitVBlankIntr();
            DnDrawManager();
            if (end_flag)
            {
                end_flag = FALSE;
                OS_TPrintf("MIC TEST stop\n");
                OS_TPrintf("push A_BUTTON to restart\n");

                ReadyCount = 0;
                OS_CancelAlarm(&sReadyAlarm);

                mictest_end();
                mem_free(s_Work);

                break;
            }
        }

        ObjEnd();
        DnBgClr();

        while(1)
        {
            u16 pad = PAD_Read();

            if (pad & PAD_BUTTON_A)
            {
                break;
            }
            if (pad & PAD_BUTTON_START)
            {
                tp_end();
                goto _release;
            }
            OS_WaitVBlankIntr();
        }

        OS_TPrintf("MIC TEST restart\n");
        fadein();
        tp_end();
    }
_release:
    if(OS_IsAlarmAvailable()){
        OS_EndAlarm();
    }
    DnObjEnd();
    FontRelease();

    OS_Terminate();
}

/*---------------------------------------------------------------------------*
    Update the progress of the standby screen
 *---------------------------------------------------------------------------*/
void ReadyUpdateCallback(void* arg)
{
#pragma unused( arg )

    BgDnMicReadyDrawText(ReadyCount);

    ReadyCount++;

    if(ReadyCount > 10)
    {
        ReadyCount = 0;
        OS_CancelAlarm(&sReadyAlarm);
    }
}

