/*---------------------------------------------------------------------------*
  Project:  MicTest - sample2
  File:     obj.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include <nnsys/g2d.h>
#include "obj.h"
#include "memory.h"
#include "cell.h"
#include "file.h"

//------------------------------------------------------------
#define  OBJ_UP_MAX    128
#define  OBJ_DN_MAX    128

#define  OBJ_BIG_X     (128-59)
#define  OBJ_BIG_Y     (96-14)
#define  OBJ_MIDDLE_X  OBJ_BIG_X
#define  OBJ_MIDDLE_Y  (96+19)
#define  OBJ_SMALL_X   OBJ_BIG_X
#define  OBJ_SMALL_Y   (96+52)
#define  OBJ_MORE_X    OBJ_BIG_X
#define  OBJ_MORE_Y    (96-39)

#define  FRAME_MAX  60

//------------------------------------------------------------
static  GXOamAttr  OamUp[OBJ_UP_MAX];   // OAM for the upper screen

static  cell  icon[4];
static  cell  modoru[2];

static  OamNum;
static  int frame = FRAME_MAX; // Wait a frame because the response will be too quick if we apply it to OBJ right away
static  int hold_lvl = -1;

//------------------------------------------------------------
void ObjInit()
{
    void*                   pPltFile;
    void*                   pChrFile;
    NNSG2dPaletteData*      pPltData;
    NNSG2dCharacterData*    pChrData;
    BOOL  ret;
    int  i;

    MI_CpuFillFast((void *)HW_OAM   , 192, HW_OAM_SIZE);
    MI_CpuFillFast((void *)HW_DB_OAM, 192, HW_DB_OAM_SIZE);

    // Mapping mode configuration  1D 128 KB
    GX_SetOBJVRamModeChar (GX_OBJVRAMMODE_CHAR_1D_128K);
    GXS_SetOBJVRamModeChar(GX_OBJVRAMMODE_CHAR_1D_128K);

    pChrFile = LoadFile("E11u_obj.NCGR");
    ret = NNS_G2dGetUnpackedCharacterData(pChrFile, &pChrData);
    SDK_ASSERT(ret);
    pPltFile = LoadFile("E11u_obj.NCLR");
    ret = NNS_G2dGetUnpackedPaletteData  (pPltFile, &pPltData);
    SDK_ASSERT(ret);

    DC_StoreRange         (pPltData->pRawData   , pPltData->szByte);
    GX_BeginLoadOBJExtPltt();
        GX_LoadOBJExtPltt (pPltData->pRawData, 0, pPltData->szByte);
    GX_EndLoadOBJExtPltt();

    DC_StoreRange (pChrData->pRawData   , pChrData->szByte);
    GX_LoadOBJ    (pChrData->pRawData, 0, pChrData->szByte);

    mem_free(pPltFile);
    mem_free(pChrFile);

    Cell_SetCellData("E11u_obj.NCER", GE_MAIN);

    // Large
    OamNum = 0;
    icon[0].celldata = RetCellData(0 , GE_MAIN);
    icon[0].x = OBJ_BIG_X; icon[0].y = OBJ_BIG_Y;
    icon[0].num =OamNum;
    OamNum += SetCellDataToOAM(&OamUp[OamNum], &icon[0]);
    // Medium
    icon[1].celldata = RetCellData(1 , GE_MAIN);
    icon[1].x = OBJ_MIDDLE_X; icon[1].y = OBJ_MIDDLE_Y;
    icon[1].num =OamNum;
    OamNum += SetCellDataToOAM(&OamUp[OamNum], &icon[1]);
    // Small
    icon[2].celldata = RetCellData(2 , GE_MAIN);
    icon[2].x = OBJ_SMALL_X; icon[2].y = OBJ_SMALL_Y;
    icon[2].num =OamNum;
    OamNum += SetCellDataToOAM(&OamUp[OamNum], &icon[2]);
    // Too large
    icon[3].celldata = RetCellData(3 , GE_MAIN);
    icon[3].x = OBJ_MORE_X; icon[3].y = OBJ_MORE_Y;
    icon[3].num =OamNum;
    OamNum += SetCellDataToOAM(&OamUp[OamNum], &icon[3]);
    SDK_ASSERT(OamNum < OBJ_UP_MAX);

    // Turn off all upper OBJ for the moment  
    for(i=0;i<OBJ_UP_MAX;i++)
    {
        G2_SetOBJEffect(&OamUp[i], GX_OAM_EFFECT_NODISPLAY, NULL);
    }

    frame = FRAME_MAX;
    hold_lvl = -1;
}

//------------------------------------------------------------
void ObjMain(const int lvl)
{
    int i;

    DC_StoreRange(OamUp,    OBJ_UP_MAX *sizeof(GXOamAttr));
    GX_LoadOAM   (OamUp, 0, OBJ_UP_MAX *sizeof(GXOamAttr));

    if((hold_lvl >= 0 && hold_lvl <= lvl) || frame < 0)
    {
        hold_lvl = lvl;
        frame = FRAME_MAX;
    }
    else
    {
        frame--;
    }

    // Turn OFF all OBJ for the moment  
    for(i=0;i<OamNum;i++)
    {
        G2_SetOBJEffect(&OamUp[i], GX_OAM_EFFECT_NODISPLAY, NULL);
    }
    if (hold_lvl>3)  // Too large
    {
        SetCellDataToOAM(&OamUp[icon[3].num], &icon[3]);
    }
    if (hold_lvl>2)  // Large
    {
        SetCellDataToOAM(&OamUp[icon[0].num], &icon[0]);
    }
    if (hold_lvl>1)  // Medium
    {
        SetCellDataToOAM(&OamUp[icon[1].num], &icon[1]);
    }
    if (hold_lvl>0)  // Small
    {
        SetCellDataToOAM(&OamUp[icon[2].num], &icon[2]);
    }
}

//------------------------------------------------------------
void ObjEnd()
{
    MI_CpuFillFast((void *)HW_OAM   , 192, HW_OAM_SIZE);
    Cell_UnloadCellFiles(GE_MAIN);
}
