/*---------------------------------------------------------------------------*
  Project:  MicTest - sample2
  File:     tp.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include "tp.h"
#include "memory.h"

#define     SAMPLING_FREQUENCE      4
#define     SAMPLING_BUFSIZE        (SAMPLING_FREQUENCE + 1)
#define     SAMPLING_START_VCOUNT   0

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
static TPData gTpBuf[SAMPLING_BUFSIZE];

static  u16  OnOffTrg;
//0 - Not pressed
//1 - Pressed
//2 - The instant it was released

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void tp_init()
{
    BOOL ret;
    TPCalibrateParam calibrate;

    TP_Init();
    OnOffTrg = TOUCH_NONE;
    ret = TP_GetUserInfo(&calibrate);
    SDK_ASSERT(ret);

    TP_SetCalibrateParam(&calibrate);
}
/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void tp_start()
{
    u32 rs = TP_RequestAutoSamplingStart(SAMPLING_START_VCOUNT, SAMPLING_FREQUENCE,
                                         gTpBuf, SAMPLING_BUFSIZE);
    SDK_ASSERT(!rs);
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
#define  ATARI_X_MIN  (128-45)
#define  ATARI_X_MAX  ((OBJ_WIDTH_DN *8) +128-45+20)
#define  ATARI_Y_MIN  (96+67-14)
#define  ATARI_Y_MAX  ((OBJ_HEIGHT_DN*8) + 96+67+14)

u16  tp_main()
{
    int i,idx;
    u32 size = SAMPLING_FREQUENCE;
    TPData  sample;
    int last_idx = TP_GetLatestIndexInAuto();

    for (i=0;i<size;i++)
    {
        idx = last_idx - (s32)size + i + 1;
        if (idx < 0)
        {
            idx += SAMPLING_BUFSIZE;
        }

        if (!gTpBuf[idx].touch)
        {
            if (OnOffTrg == TOUCH_ON_TRG)
            {
                OnOffTrg = TOUCH_NONE;
                return TOUCH_OFF_TRG;
            }
            continue;
        }

        if (gTpBuf[idx].validity != TP_VALIDITY_VALID)
        {
            continue;
        }

        TP_GetCalibratedPoint(&sample, &gTpBuf[idx]);

        if (sample.x >= ATARI_X_MIN && sample.x <= ATARI_X_MAX &&
            sample.y >= ATARI_Y_MIN && sample.y <= ATARI_Y_MAX)
        {
            OnOffTrg = TOUCH_ON_TRG;
            return OnOffTrg;
        }
    }
    return  TOUCH_NONE;
}

/*---------------------------------------------------------------------------*
 *---------------------------------------------------------------------------*/
void tp_end()
{
    u32 rs = TP_RequestAutoSamplingStop();
    SDK_ASSERT(!rs);
}
