/*---------------------------------------------------------------------------*
  Project:  MicTest
  File:     MicTest.c

  Copyright 2003-2008 Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include <nitro.h>
#include "MicTest.h"

#define  LEVEL1  (150)
#define  LEVEL2  (162)
#define  LEVEL3  (176)
#define  LEVEL4  (197)

static u8* sRecBuffer;
static MICAutoParam sMicParam;
static OSAlarm sAlarm;
static u8 sIsReady;       // Mic circuit stability flag


/*---------------------------------------------------------------------------*
    Initialization
 *---------------------------------------------------------------------------*/
void mictest_init(const void* buffer)
{
    SDK_ASSERT(buffer);
    MIC_Init();
    PM_Init();
// AMP on
    (void)PM_SetAmp( PM_AMP_ON );
// Adjust AMP gain
    (void)PM_SetAmpGain( PM_AMPGAIN_80 );
    sRecBuffer = (u8*)buffer;
    sMicParam.type = MIC_SAMPLING_TYPE_8BIT;
    sMicParam.buffer = sRecBuffer;
    sMicParam.size = MICTEST_SIZE;
    sMicParam.rate = MIC_SAMPLING_RATE_8K;
    sMicParam.loop_enable = TRUE;
    sMicParam.full_callback = NULL;
    sMicParam.full_arg = NULL;
    
// Alarm settings
    sIsReady = 0;
    OS_CreateAlarm(&sAlarm);
    OS_SetAlarm(&sAlarm, MICTEST_READY_TICKS, &mictest_initialCheckCallback, NULL);
}

/*---------------------------------------------------------------------------*
    Start
 *---------------------------------------------------------------------------*/
void mictest_start(void)
{
    (void)MIC_StartAutoSampling(&sMicParam);
}

/*---------------------------------------------------------------------------*
    Each frame
 *---------------------------------------------------------------------------*/
int mictest_main(void)
{
    u32 i;
    u8  j;
    u8  max=0;
    
    if(!sIsReady)
    {
        return 0;
    }
    
    DC_InvalidateRange(sRecBuffer, MICTEST_SIZE);
    
    for(i=0;i<MICTEST_SIZE;i++)
    {
        j = sRecBuffer[i];
    // Level restrictions (check the range guaranteed for microphone input values)
        if (j < 28 ) continue;
        if (j > 227) continue;

        if (j < 128) j = (u8)(256 - sRecBuffer[i]);
        if (max < j) max = j;
    }
    if      (max > LEVEL4) return 4;
    else if (max > LEVEL3) return 3;
    else if (max > LEVEL2) return 2;
    else if (max > LEVEL1) return 1;
    else return 0;
}

/*---------------------------------------------------------------------------*
    Quit
 *---------------------------------------------------------------------------*/
void mictest_end(void)
{
    if(!sIsReady)
    {
        OS_CancelAlarm(&sAlarm);
    }
    
    (void)MIC_StopAutoSampling();
    (void)PM_SetAmp( PM_AMP_OFF );
}


/*---------------------------------------------------------------------------*
    Mic circuit stability notification (alarm callback)
 *---------------------------------------------------------------------------*/
void mictest_initialCheckCallback(void* arg)
{
#pragma unused( arg )
    OS_Printf("****** MIC is READY *******I\n");
    sIsReady = 1;
    OS_CancelAlarm(&sAlarm);
}

/*---------------------------------------------------------------------------*
    Get the mic circuit status
 *---------------------------------------------------------------------------*/
u8 mictest_IsReady(void)
{
    return sIsReady;
}
